@@ -28,6 +28,10 @@ struct temp_event {
   struct temp_event *next;
 };
 
+// The following macro follows if a string function has an error. It should 
+// never happen; but it is necessary to prevent compiler warnings. We print 
+// something just in case there is programmer error in invoking the function.
+#define HANDLE_STRING_ERROR {fprintf(stderr,"%s:%i unexpected string function error.\n",__FILE__,__LINE__); exit(-1);}
 
 static CORETEMP_native_event_entry_t * _coretemp_native_events;
 static int num_events		= 0;
@@ -125,9 +129,12 @@ generateEventList(char *base_dir)
 	 /* Sometimes the files are in ./, sometimes in device/ */
 	 for(pathnum=0;pathnum<NUM_PATHS;pathnum++) {
 
-	    snprintf(path, PATH_MAX, "%s/%s/%s", 
+	    retlen = snprintf(path, PATH_MAX, "%s/%s/%s",
 		     base_dir, hwmonx->d_name,paths[pathnum]);
-
+        if (retlen <= 0 || PATH_MAX <= retlen) {
+            SUBDBG("Path length is too long.\n");
+            return PAPI_EINVAL;
+        }
 	    SUBDBG("Trying to open %s\n",path);
 	    d = opendir(path);
 	    if (d==NULL) {
@@ -136,7 +143,11 @@ generateEventList(char *base_dir)
 
 	    /* Get the name of the module */
 
-	    snprintf(filename, PAPI_MAX_STR_LEN, "%s/name",path);
+	    retlen = snprintf(filename, PAPI_MAX_STR_LEN, "%s/name",path);
+        if (retlen <= 0 || PAPI_MAX_STR_LEN <= retlen) {
+            SUBDBG("Module name too long.\n");
+            return PAPI_EINVAL;
+        }
 	    fff=fopen(filename,"r");
 	    if (fff==NULL) {
 	       snprintf(modulename, PAPI_MIN_STR_LEN, "Unknown");
@@ -161,8 +172,12 @@ generateEventList(char *base_dir)
 	  for(i=0;i<32;i++) {
 
 	     /* Try looking for a location label */
-	     snprintf(filename, PAPI_MAX_STR_LEN, "%s/in%d_label", 
+	     retlen = snprintf(filename, PAPI_MAX_STR_LEN, "%s/in%d_label",
 		      path,i);
+         if (retlen <= 0 || PAPI_MAX_STR_LEN <= retlen) {
+            SUBDBG("Failed to construct location label.\n");
+            return PAPI_EINVAL;
+         }
 	     fff=fopen(filename,"r");
 	     if (fff==NULL) {
 	        strncpy(location,"?",PAPI_MIN_STR_LEN);
@@ -175,8 +190,12 @@ generateEventList(char *base_dir)
 	     }
 
 	     /* Look for input temperature */
-	     snprintf(filename, PAPI_MAX_STR_LEN, "%s/in%d_input", 
+	     retlen = snprintf(filename, PAPI_MAX_STR_LEN, "%s/in%d_input",
 		      path,i);
+         if (retlen <= 0 || PAPI_MAX_STR_LEN <= retlen) {
+            SUBDBG("Failed input temperature string.\n");
+            return PAPI_EINVAL;
+         }
 	     fff=fopen(filename,"r");
 	     if (fff==NULL) continue;
 	     fclose(fff);
@@ -184,14 +203,19 @@ generateEventList(char *base_dir)
 	     retlen = snprintf(name, PAPI_MAX_STR_LEN, "%s:in%i_input", hwmonx->d_name, i);
 	     if (retlen <= 0 || PAPI_MAX_STR_LEN <= retlen) {
 	         SUBDBG("Unable to generate name %s:in%i_input\n", hwmonx->d_name, i);
-            closedir(d);
+		 closedir(dir);
+		 closedir(d);
 	         return ( PAPI_EINVAL );
 	     }
 
 	     snprintf(units, PAPI_MIN_STR_LEN, "V");
-	     snprintf(description, PAPI_MAX_STR_LEN, "%s, %s module, label %s",
+	     retlen = snprintf(description, PAPI_MAX_STR_LEN, "%s, %s module, label %s",
 		      units,modulename,
 		      location);
+         if (retlen <= 0 || PAPI_MAX_STR_LEN <= retlen) {
+            SUBDBG("snprintf failed.\n");
+            return PAPI_EINVAL;
+         }
 
 	     if (insert_in_list(name,units,description,filename)!=PAPI_OK) {
 	        goto done_error;
@@ -208,8 +232,12 @@ generateEventList(char *base_dir)
 	  for(i=0;i<32;i++) {
 
 	     /* Try looking for a location label */
-	     snprintf(filename, PAPI_MAX_STR_LEN, "%s/temp%d_label", 
+	     retlen = snprintf(filename, PAPI_MAX_STR_LEN, "%s/temp%d_label",
 		      path,i);
+         if (retlen <= 0 || PAPI_MAX_STR_LEN <= retlen) {
+            SUBDBG("Location label string failed.\n");
+            return PAPI_EINVAL;
+         }
 	     fff=fopen(filename,"r");
 	     if (fff==NULL) {
 	        strncpy(location,"?",PAPI_MIN_STR_LEN);
@@ -222,8 +250,12 @@ generateEventList(char *base_dir)
 	     }
 
 	     /* Look for input temperature */
-	     snprintf(filename, PAPI_MAX_STR_LEN, "%s/temp%d_input", 
+	     retlen = snprintf(filename, PAPI_MAX_STR_LEN, "%s/temp%d_input",
 		      path,i);
+         if (retlen <= 0 || PAPI_MAX_STR_LEN <= retlen) {
+            SUBDBG("Input temperature string failed.\n");
+            return PAPI_EINVAL;
+         }
 	     fff=fopen(filename,"r");
 	     if (fff==NULL) continue;
 	     fclose(fff);
@@ -237,9 +269,13 @@ generateEventList(char *base_dir)
 	     }
 
 	     snprintf(units, PAPI_MIN_STR_LEN, "degrees C");
-	     snprintf(description, PAPI_MAX_STR_LEN, "%s, %s module, label %s",
+	     retlen = snprintf(description, PAPI_MAX_STR_LEN, "%s, %s module, label %s",
 		      units,modulename,
 		      location);
+         if (retlen <= 0 || PAPI_MAX_STR_LEN <= retlen) {
+            SUBDBG("snprintf failed.\n");
+            return PAPI_EINVAL;
+         }
 
 	     if (insert_in_list(name,units,description,filename)!=PAPI_OK) {
 	        goto done_error;
@@ -255,8 +291,12 @@ generateEventList(char *base_dir)
 	  for(i=0;i<32;i++) {
 
 	     /* Try looking for a location label */
-	     snprintf(filename, PAPI_MAX_STR_LEN, "%s/fan%d_label", 
+	     retlen = snprintf(filename, PAPI_MAX_STR_LEN, "%s/fan%d_label",
 		      path,i);
+         if (retlen <= 0 || PAPI_MAX_STR_LEN <= retlen) {
+            SUBDBG("Failed to write fan label string.\n");
+            return PAPI_EINVAL;
+         }
 	     fff=fopen(filename,"r");
 	     if (fff==NULL) {
 	        strncpy(location,"?",PAPI_MIN_STR_LEN);
@@ -290,9 +330,13 @@ generateEventList(char *base_dir)
 	     }
 
 	     snprintf(units, PAPI_MIN_STR_LEN, "RPM");
-	     snprintf(description, PAPI_MAX_STR_LEN, "%s, %s module, label %s",
+	     retlen = snprintf(description, PAPI_MAX_STR_LEN, "%s, %s module, label %s",
 		      units,modulename,
 		      location);
+         if (retlen <= 0 || PAPI_MAX_STR_LEN <= retlen) {
+            SUBDBG("snprintf failed.\n");
+            return PAPI_EINVAL;
+         }
 
 	     if (insert_in_list(name,units,description,filename)!=PAPI_OK) {
 	        goto done_error;
@@ -365,11 +409,12 @@ _coretemp_init_thread( hwd_context_t *ctx )
 static int
 _coretemp_init_component( int cidx )
 {
+    int retval = PAPI_OK;
      int i = 0;
      struct temp_event *t,*last;
 
      if ( is_initialized )
-	return (PAPI_OK );
+         goto fn_exit;
 
      is_initialized = 1;
 
@@ -379,37 +424,56 @@ _coretemp_init_component( int cidx )
      num_events = generateEventList("/sys/class/hwmon");
 
      if ( num_events < 0 ) {
-        strncpy(_coretemp_vector.cmp_info.disabled_reason,
-		"Cannot open /sys/class/hwmon",PAPI_MAX_STR_LEN);
-	return PAPI_ENOCMP;
+        char* strCpy;
+        strCpy=strncpy(_coretemp_vector.cmp_info.disabled_reason,
+        "Cannot open /sys/class/hwmon",PAPI_MAX_STR_LEN);
+        _coretemp_vector.cmp_info.disabled_reason[PAPI_MAX_STR_LEN-1]=0;
+        if (strCpy == NULL) HANDLE_STRING_ERROR;
+        retval = PAPI_ECMP;
+        goto fn_fail;
      }
 
      if ( num_events == 0 ) {
-        strncpy(_coretemp_vector.cmp_info.disabled_reason,
-		"No coretemp events found",PAPI_MAX_STR_LEN);
-	return PAPI_ENOCMP;
+        char* strCpy=strncpy(_coretemp_vector.cmp_info.disabled_reason,
+        "No coretemp events found",PAPI_MAX_STR_LEN);
+        _coretemp_vector.cmp_info.disabled_reason[PAPI_MAX_STR_LEN-1]=0;
+        if (strCpy == NULL) HANDLE_STRING_ERROR;
+        retval = PAPI_ECMP;
+        goto fn_fail;
      }
 
      t = root;
   
      _coretemp_native_events = (CORETEMP_native_event_entry_t*)
           papi_calloc(num_events, sizeof(CORETEMP_native_event_entry_t));
+     if (_coretemp_native_events == NULL) {
+          int strErr=snprintf(_coretemp_vector.cmp_info.disabled_reason, PAPI_MAX_STR_LEN, "malloc() of _coretemp_native_events failed for %lu bytes.", num_events*sizeof(CORETEMP_native_event_entry_t));
+          _coretemp_vector.cmp_info.disabled_reason[PAPI_MAX_STR_LEN-1]=0;
+          if (strErr > PAPI_MAX_STR_LEN) HANDLE_STRING_ERROR;
+          retval = PAPI_ENOMEM;
+          goto fn_fail;
+     }
 
      do {
-	strncpy(_coretemp_native_events[i].name,t->name,PAPI_MAX_STR_LEN);
-        _coretemp_native_events[i].name[PAPI_MAX_STR_LEN-1] = '\0';
-	strncpy(_coretemp_native_events[i].path,t->path,PATH_MAX);
-        _coretemp_native_events[i].path[PATH_MAX-1] = '\0';
-	strncpy(_coretemp_native_events[i].units,t->units,PAPI_MIN_STR_LEN);
-	_coretemp_native_events[i].units[PAPI_MIN_STR_LEN-1] = '\0';
-	strncpy(_coretemp_native_events[i].description,t->description,PAPI_MAX_STR_LEN);
-        _coretemp_native_events[i].description[PAPI_MAX_STR_LEN-1] = '\0';
-	_coretemp_native_events[i].stone = 0;
-	_coretemp_native_events[i].resources.selector = i + 1;
-	last	= t;
-	t		= t->next;
-	papi_free(last);
-	i++;
+        int retlen;
+        retlen = snprintf(_coretemp_native_events[i].name, PAPI_MAX_STR_LEN, "%s", t->name);
+        if (retlen <= 0 || retlen >= PAPI_MAX_STR_LEN) HANDLE_STRING_ERROR;
+
+        retlen = snprintf(_coretemp_native_events[i].path, PATH_MAX, "%s", t->path);
+        if (retlen <= 0 || retlen >= PATH_MAX) HANDLE_STRING_ERROR;
+
+        retlen = snprintf(_coretemp_native_events[i].units, PAPI_MIN_STR_LEN, "%s", t->units);
+        if (retlen <= 0 || retlen >= PAPI_MIN_STR_LEN) HANDLE_STRING_ERROR;
+
+        retlen = snprintf(_coretemp_native_events[i].description, PAPI_MAX_STR_LEN, "%s",t->description);
+        if (retlen <= 0 || retlen >= PAPI_MAX_STR_LEN) HANDLE_STRING_ERROR;
+
+	    _coretemp_native_events[i].stone = 0;
+	    _coretemp_native_events[i].resources.selector = i + 1;
+	    last	= t;
+	    t		= t->next;
+	    papi_free(last);
+	    i++;
      } while (t != NULL);
      root = NULL;
 
@@ -419,7 +483,11 @@ _coretemp_init_component( int cidx )
      /* Export the component id */
      _coretemp_vector.cmp_info.CmpIdx = cidx;
 
-     return PAPI_OK;
+  fn_exit:
+     _papi_hwd[cidx]->cmp_info.disabled = retval;
+     return retval;
+  fn_fail:
+     goto fn_exit;
 }
 
 
