@@ -116,6 +116,18 @@ static const sensors_subfeature *(*sensors_get_all_subfeaturesPtr)(const sensors
 // file handles used to access lmsensors libraries with dlopen
 static void* dl1 = NULL;
 
+// string macro defined within Rules.lmsensors
+static char lmsensors_main[]=PAPI_LMSENSORS_MAIN;
+
+//-----------------------------------------------------------------------------
+// Using weak symbols (global declared without a value, so it defers to any
+// other global declared in another file WITH a value) allows PAPI to be built
+// with the component, but PAPI can still be installed in a system without the
+// required library.
+//-----------------------------------------------------------------------------
+
+void (*_dl_non_dynamic_init)(void) __attribute__((weak));               // declare a weak dynamic-library init routine pointer.
+
 static int link_lmsensors_libraries ();
 
 papi_vector_t _lmsensors_vector;
@@ -159,14 +171,11 @@ static unsigned
 createNativeEvents( void )
 {
 	unsigned id = 0;
-	unsigned int count;
-	(void) count; // Ignore not used warnings.
-
 	int chip_nr = 0;
 	const sensors_chip_name *chip_name;
 
 	/* component name and description */
-	strcpy( _lmsensors_vector.cmp_info.short_name, "lm_sensors" );
+	strcpy( _lmsensors_vector.cmp_info.short_name, "lmsensors" );
 	strcpy( _lmsensors_vector.cmp_info.description,
 			"lm-sensors provides tools for monitoring the hardware health" );
 
@@ -202,7 +211,9 @@ createNativeEvents( void )
 	      /* Loop through all the subfeatures found */
 	      while ((sub=sensors_get_all_subfeaturesPtr(chip_name,feature,&b))) {
 
-	         count = 0;
+		 /* replace spaces with underscores */
+		 char *ptr;
+		 while( (ptr = strchr(featurelabel, ' ')) != NULL ){ *ptr = '_'; }
 
 		 /* Save native event data */
 		 sprintf( lm_sensors_native_table[id].name, "%s.%s.%s",
@@ -219,8 +230,6 @@ createNativeEvents( void )
 		 lm_sensors_native_table[id].resources.name = chip_name;
 		 lm_sensors_native_table[id].resources.subfeat_nr = sub->number;
 
-		 count = sub->number;
-
 		 /* increment the table index counter */
 		 id++;		 
 	      }
@@ -289,14 +298,15 @@ _lmsensors_init_component( int cidx )
     if (link_lmsensors_libraries() != PAPI_OK) {
 	    SUBDBG ("Dynamic link of lmsensors libraries failed, component will be disabled.\n");
 	    SUBDBG ("See disable reason in papi_component_avail output for more details.\n");
-	    return (PAPI_ENOSUPP);
+        res = PAPI_ENOSUPP;
+        goto fn_fail;
     }
 
     /* Initialize libsensors library */
     if ( ( res = sensors_initPtr( NULL ) ) != 0 ) {
        strncpy(_lmsensors_vector.cmp_info.disabled_reason,
 	      "Cannot enable libsensors",PAPI_MAX_STR_LEN);
-       return res;
+       goto fn_fail;
     }
 
     /* Create dyanmic events table */
@@ -311,7 +321,8 @@ _lmsensors_init_component( int cidx )
 				   == NULL ) {
        strncpy(_lmsensors_vector.cmp_info.disabled_reason,
 	      "Could not malloc room",PAPI_MAX_STR_LEN);
-       return PAPI_ENOMEM;
+       res = PAPI_ENOMEM;
+       goto fn_fail;
     }
 
     cached_counts = (long long*) calloc(num_events, sizeof(long long));
@@ -321,7 +332,8 @@ _lmsensors_init_component( int cidx )
                "Could not malloc room",PAPI_MAX_STR_LEN);
       free(lm_sensors_native_table);
       lm_sensors_native_table = NULL;
-	   return PAPI_ENOMEM;
+      res = PAPI_ENOMEM;
+      goto fn_fail;
     }
 
     if ( ( unsigned ) num_events != createNativeEvents(  ) ) {
@@ -331,13 +343,18 @@ _lmsensors_init_component( int cidx )
       cached_counts = NULL;
       free(lm_sensors_native_table);
       lm_sensors_native_table = NULL;
-       return PAPI_ECMP;
+      res = PAPI_ECMP;
+      goto fn_fail;
     }
 
     _lmsensors_vector.cmp_info.num_native_events=num_events;
     _lmsensors_vector.cmp_info.num_cntrs=num_events;
 
-    return PAPI_OK;
+  fn_exit:
+    _papi_hwd[cidx]->cmp_info.disabled = res;
+    return res;
+  fn_fail:
+    goto fn_exit;
 }
 
 /*
@@ -351,14 +368,51 @@ _lmsensors_init_component( int cidx )
 static int
 link_lmsensors_libraries ()
 {
-	/* Need to link in the lmsensors libraries, if not found disable the component */
-	dl1 = dlopen("libsensors.so", RTLD_NOW | RTLD_GLOBAL);
-	if (!dl1)
-	{
-		strncpy(_lmsensors_vector.cmp_info.disabled_reason,
-			"lmsensors library libsensors.so not found.",PAPI_MAX_STR_LEN);
-		return ( PAPI_ENOSUPP );
-	}
+   if ( _dl_non_dynamic_init != NULL ) {
+      // If weak var present, statically linked insted of dynamic.
+      strncpy( _lmsensors_vector.cmp_info.disabled_reason, "The lmsensors component REQUIRES dynamic linking capabilities.", PAPI_MAX_STR_LEN-1);
+      // EXIT not supported.
+      return PAPI_ENOSUPP;
+   }
+
+   char path_name[1024];
+   char *lmsensors_root = getenv("PAPI_LMSENSORS_ROOT"); 
+   
+   dl1 = NULL;
+   // Step 1: Process override if given.   
+   if (strlen(lmsensors_main) > 0) {                                  // If override given, it has to work.
+      dl1 = dlopen(lmsensors_main, RTLD_NOW | RTLD_GLOBAL);           // Try to open that path.
+      if (dl1 == NULL) {
+         snprintf(_lmsensors_vector.cmp_info.disabled_reason, PAPI_MAX_STR_LEN, "PAPI_LMSENSORS_MAIN override '%s' given in Rules.lmsensors not found.", lmsensors_main);
+         return(PAPI_ENOSUPP);   // Override given but not found.
+      }
+   }
+
+   // Step 2: Try system paths, will work with Spack, LD_LIBRARY_PATH, default paths.
+   if (dl1 == NULL) {                                           // No override,
+      dl1 = dlopen("libsensors.so", RTLD_NOW | RTLD_GLOBAL);        // Try system paths.
+   }
+
+   // Step 3: Try the explicit install default. 
+   if (dl1 == NULL && lmsensors_root != NULL) {                          // if root given, try it.
+      snprintf(path_name, PATH_MAX, "%s/lib64/libsensors.so", lmsensors_root);   // PAPI Root check.
+      dl1 = dlopen(path_name, RTLD_NOW | RTLD_GLOBAL);             // Try to open that path.
+   }
+
+   // Step 4: Try another explicit install default.
+   if (dl1 == NULL && lmsensors_root != NULL) {                          // if root given, try it.
+      snprintf(path_name, PATH_MAX, "%s/lib/libsensors.so", lmsensors_root);   // PAPI Root check.
+      dl1 = dlopen(path_name, RTLD_NOW | RTLD_GLOBAL);             // Try to open that path.
+   }
+
+   // Check for failure.
+   if (dl1 == NULL) {
+      snprintf(_lmsensors_vector.cmp_info.disabled_reason, PAPI_MAX_STR_LEN, "libsensors.so not found.");
+      return(PAPI_ENOSUPP);
+   }
+
+   // We have dl1. 
+
 	sensors_initPtr = dlsym(dl1, "sensors_init");
 	if (dlerror() != NULL)
 	{
