@@ -51,6 +51,11 @@ papi_vector_t _net_vector;
 #define NET_INVALID_RESULT     -1
 
 
+// The following macro follows if a string function has an error. It should 
+// never happen; but it is necessary to prevent compiler warnings. We print 
+// something just in case there is programmer error in invoking the function.
+#define HANDLE_STRING_ERROR {fprintf(stderr,"%s:%i unexpected string function error.\n",__FILE__,__LINE__); exit(-1);}
+
 static NET_native_event_entry_t * _net_native_events=NULL;
 
 static int num_events       = 0;
@@ -117,6 +122,9 @@ generateNetEventList( void )
     if (fin == NULL) {
         SUBDBG("Can't find %s, are you sure the /proc file-system is mounted?\n",
            NET_PROC_FILE);
+        snprintf(_net_vector.cmp_info.disabled_reason, PAPI_MAX_STR_LEN-2,
+        "Failed to find /proc file-system.");
+        _net_vector.cmp_info.disabled_reason[PAPI_MAX_STR_LEN-1]=0;    // force null termination.
         return 0;
     }
 
@@ -126,6 +134,9 @@ generateNetEventList( void )
         if (retval == NULL) {
         	fclose(fin);
             SUBDBG("Not enough lines in %s\n", NET_PROC_FILE);
+            snprintf(_net_vector.cmp_info.disabled_reason, PAPI_MAX_STR_LEN-2,
+            "Not enough lines in '%s'.", NET_PROC_FILE);
+            _net_vector.cmp_info.disabled_reason[PAPI_MAX_STR_LEN-1]=0;    // force null termination.
             return 0;
         }
     }
@@ -150,6 +161,9 @@ generateNetEventList( void )
             if (!temp) {
                 PAPIERROR("out of memory!");
                 fclose(fin);
+                snprintf(_net_vector.cmp_info.disabled_reason, PAPI_MAX_STR_LEN-2,
+                "%s failed to allocate %lu bytes for temp_event.", __func__, sizeof(struct temp_event));
+                _net_vector.cmp_info.disabled_reason[PAPI_MAX_STR_LEN-1]=0;    // force null termination.
                 return PAPI_ENOMEM;
             }
             temp->next = NULL;
@@ -162,13 +176,20 @@ generateNetEventList( void )
                 free(temp);
                 fclose(fin);
                 PAPIERROR("This shouldn't be possible\n");
+                snprintf(_net_vector.cmp_info.disabled_reason, PAPI_MAX_STR_LEN-2,
+                "%s Error navigating linked list.", __func__);
+                _net_vector.cmp_info.disabled_reason[PAPI_MAX_STR_LEN-1]=0;    // force null termination.
                 return PAPI_ECMP;
             }
             last = temp;
 
-            snprintf(temp->name, PAPI_MAX_STR_LEN, "%s:%s",
+            size_t str_len = strlen(ifname) + strlen(_net_counter_info[j].name) + 1;
+            str_len = (str_len > PAPI_MAX_STR_LEN - 1) ? PAPI_MAX_STR_LEN - 1 : str_len;
+            snprintf(temp->name, str_len, "%s:%s",
                     ifname, _net_counter_info[j].name);
-            snprintf(temp->description, PAPI_MAX_STR_LEN, "%s %s",
+            str_len = strlen(ifname) + strlen(_net_counter_info[j].description) + 1;
+            str_len = (str_len > PAPI_MAX_STR_LEN - 1) ? PAPI_MAX_STR_LEN - 1 : str_len;
+            snprintf(temp->description, str_len, "%s %s",
                     ifname, _net_counter_info[j].description);
 
             count++;
@@ -298,11 +319,12 @@ _net_init_thread( hwd_context_t *ctx )
 static int
 _net_init_component( int cidx  )
 {
+    int retval = PAPI_OK;
     int i = 0;
     struct temp_event *t, *last;
 
     if ( is_initialized )
-        return PAPI_OK;
+        goto fn_exit;
 
     memset(_net_register_start, 0,
     		NET_MAX_COUNTERS*sizeof(_net_register_start[0]));
@@ -314,20 +336,23 @@ _net_init_component( int cidx  )
     /* The network interfaces are listed in /proc/net/dev */
     num_events = generateNetEventList();
 
-    if ( num_events < 0 )  /* PAPI errors */
-        return num_events;
+    if ( num_events < 0 ) {  /* disabled_reason already set */
+        retval = PAPI_ECMP;
+        goto fn_fail;
+    }
 
     if ( num_events == 0 )  /* No network interfaces found */
-        return PAPI_OK;
+        goto fn_exit;
 
     t = root;
     _net_native_events = (NET_native_event_entry_t*)
         papi_malloc(sizeof(NET_native_event_entry_t) * num_events);
     do {
-        strncpy(_net_native_events[i].name, t->name, PAPI_MAX_STR_LEN-1);
-        _net_native_events[i].name[PAPI_MAX_STR_LEN-1] = '\0';
-        strncpy(_net_native_events[i].description, t->description, PAPI_MAX_STR_LEN-1);
-        _net_native_events[i].description[PAPI_MAX_STR_LEN-1] = '\0';
+        int retlen;
+        retlen = snprintf(_net_native_events[i].name, PAPI_MAX_STR_LEN, "%s", t->name);
+        if (retlen <= 0 || retlen >= PAPI_MAX_STR_LEN) HANDLE_STRING_ERROR;
+        retlen = snprintf(_net_native_events[i].description, PAPI_MAX_STR_LEN, "%s", t->description);
+        if (retlen <= 0 || retlen >= PAPI_MAX_STR_LEN) HANDLE_STRING_ERROR;
         _net_native_events[i].resources.selector = i + 1;
         last    = t;
         t       = t->next;
@@ -342,7 +367,11 @@ _net_init_component( int cidx  )
     /* Export the component id */
     _net_vector.cmp_info.CmpIdx = cidx;
 
-    return PAPI_OK;
+  fn_exit:
+    _papi_hwd[cidx]->cmp_info.disabled = retval;
+    return retval;
+  fn_fail:
+    goto fn_exit;
 }
 
 
