@@ -31,6 +31,9 @@
 // used to step through the attributes when enumerating events
 static int attr_idx;
 
+/* alias flags to handle amd_fam17h, amd_fam17h_zen1 both present PMUs*/
+static int amd64_fam17h_zen1_present = 0;
+
 /** @class  find_existing_event
  *  @brief  looks up an event, returns it if it exists
  *
@@ -188,6 +191,9 @@ static struct native_event_t *allocate_native_event(
 	perf_arg.attr=&ntv_evt->attr;
 	perf_arg.fstr=&event_string;
 
+	// set the size of the perf attr struct before getting pfm encoding
+	ntv_evt->attr.size = sizeof(struct perf_event_attr);
+
 	/* use user provided name of the event to get the */
 	/* perf_event encoding and a fully qualified event string */
 	ret = pfm_get_os_event_encoding(name,
@@ -222,6 +228,8 @@ static struct native_event_t *allocate_native_event(
 		//return NULL;
 	}
 
+	// free string allocated by pfm_get_os_event_encoding
+	free(*(perf_arg.fstr));
 	// get a copy of the event name and break it up into its parts
 	event_string = strdup(name);
 
@@ -353,9 +361,10 @@ static struct native_event_t *allocate_native_event(
 				ret = pfm_get_event_attr_info(libpfm4_index,
 					i, PFM_OS_PERF_EVENT_EXT, &ainfo);
 				if (ret != PFM_SUCCESS) {
-					free (msk_ptr);
-					SUBDBG("EXIT: Attribute info not found, libpfm4_index: %#x, ret: %d\n", libpfm4_index, _papi_libpfm4_error(ret));
-					// FIXME: do we need to unlock here? --vmw
+               free (msk_ptr);
+               free(pmu_name);
+					SUBDBG("EXIT: error libpfm4 find event: Attribute info not found, libpfm4_index: %#x, ret: %d\n", libpfm4_index, _papi_libpfm4_error(ret));
+               _papi_hwi_unlock( NAMELIB_LOCK );
 					return NULL;
 				}
 
@@ -391,8 +400,10 @@ static struct native_event_t *allocate_native_event(
 
 			/* See if we had a mask that wasn't found */
 			if (!mask_found) {
-				SUBDBG("Mask not found! %s\n",ptr);
-				/* FIXME: do we need to unlock here? */
+            free(msk_ptr);
+            free(pmu_name);
+            SUBDBG("EXIT: error libpfm4 find event: Mask not found: %s.\n", ptr);
+            _papi_hwi_unlock( NAMELIB_LOCK );
 				return NULL;
 			}
 
@@ -479,7 +490,13 @@ static struct native_event_t *allocate_native_event(
  *
  *  @returns returns a libpfm event number
  *  @retval PAPI_ENOEVENT  Could not find an event
- *
+ *  Operational note: _pe_libpfm4_init() must be called first to set
+ *                    flags for synonymous PMUs. At this writing only 
+ *                    amd64_fam17h_zen1_present is defined.
+ *  Operational note: We indirectly return the pmu_idx within the
+ *                    event data; the calling code uses that to set
+ *                    pmu_idx for subsequent calls. All we do is find
+ *                    the next valid pmu, if any.
  */
 
 static int
@@ -508,6 +525,12 @@ get_first_event_next_pmu(int pmu_idx, int pmu_type)
 		break;
 	}
 
+    if ((ret==PFM_SUCCESS) && amd64_fam17h_zen1_present && strcmp(pinfo.name, "amd64_fam17h") == 0) {
+        /* Skip as if invalid; we want the PMU amd64_fam17h_zen1 instead. */
+        pmu_idx++;
+        continue;
+    }
+        
     if ((ret==PFM_SUCCESS) && pmu_is_present_and_right_type(&pinfo,pmu_type)) {
 
       pidx=pinfo.first_event;
@@ -1131,6 +1154,7 @@ _pe_libpfm4_init(papi_vector_t *component, int cidx,
 
 	pfm_err_t retval = PFM_SUCCESS;
 	pfm_pmu_info_t pinfo;
+	unsigned int strSize;
 
 	/* allocate the native event structure */
 	event_table->num_native_events=0;
@@ -1139,8 +1163,6 @@ _pe_libpfm4_init(papi_vector_t *component, int cidx,
 	event_table->native_events=calloc(NATIVE_EVENT_CHUNK,
 					sizeof(struct native_event_t));
 	if (event_table->native_events==NULL) {
-		strncpy(component->cmp_info.disabled_reason,
-			"calloc NATIVE_EVENT_CHUNK failed",PAPI_MAX_STR_LEN);
 		return PAPI_ENOMEM;
 	}
 
@@ -1156,6 +1178,35 @@ _pe_libpfm4_init(papi_vector_t *component, int cidx,
 	event_table->default_pmu.size = sizeof(pfm_pmu_info_t);
 	retval=pfm_get_pmu_info(0, &(event_table->default_pmu));
 
+    SUBDBG("Prescan for aliases.\n");
+    /* We have to see if we have aliases in there as separate PMUs, */
+    /* we don't want both PMUs with all the events duplicated.      */
+    /* For aliases, either is valid alone, but if both are present  */
+    /* specify a preference in the code.                            */
+    /* Alias: amd64_fam17h_zen1 over amd64_fam17h.                  */
+    /* Alias flags are static ints global to this file.             */
+    i=0;
+	while(1) {
+		memset(&pinfo,0,sizeof(pfm_pmu_info_t));
+		pinfo.size = sizeof(pfm_pmu_info_t);
+		retval=pfm_get_pmu_info(i, &pinfo);
+
+		/* We're done if we hit an invalid PMU entry		    */
+		/* We can't check against PFM_PMU_MAX as that might not	*/
+		/* match if libpfm4 is dynamically linked		        */
+
+		if (retval==PFM_ERR_INVAL) {
+			break;
+		}
+
+		if ( (retval==PFM_SUCCESS) && (pinfo.name != NULL) &&
+			(pmu_is_present_and_right_type(&pinfo,pmu_type)) &&
+            (strcmp(pinfo.name,"amd64_fam17h_zen1") == 0) ) {
+            amd64_fam17h_zen1_present = 1;
+        }
+        i++;
+    } 
+
 	SUBDBG("Detected pmus:\n");
 	i=0;
 	while(1) {
@@ -1174,6 +1225,12 @@ _pe_libpfm4_init(papi_vector_t *component, int cidx,
 		if ((retval==PFM_SUCCESS) && (pinfo.name != NULL) &&
 			(pmu_is_present_and_right_type(&pinfo,pmu_type))) {
 
+            /* skip if it is amd64_fam17h and zen1 is also present. */
+            if (strcmp(pinfo.name,"amd64_fam17h") == 0 && amd64_fam17h_zen1_present) {
+                i++;
+                continue;
+            }
+
 			SUBDBG("\t%d %s %s %d\n",i,
 				pinfo.name,pinfo.desc,pinfo.type);
 
@@ -1190,11 +1247,18 @@ _pe_libpfm4_init(papi_vector_t *component, int cidx,
 				/* Hack to have "default core" PMU */
 				if ( (pinfo.type==PFM_PMU_TYPE_CORE) &&
 					strcmp(pinfo.name,"ix86arch")) {
+					    memcpy(&(event_table->default_pmu),
+						    &pinfo,sizeof(pfm_pmu_info_t));
+                        found_default++;
+				}
 
-					SUBDBG("\t  %s is default\n",pinfo.name);
-					memcpy(&(event_table->default_pmu),
-						&pinfo,sizeof(pfm_pmu_info_t));
-					found_default++;
+				/* For ARM processors, */
+				if ( (pinfo.type==PFM_PMU_TYPE_CORE) &&
+					( _papi_hwi_system_info.hw_info.vendor >= PAPI_VENDOR_ARM_ARM)) {
+					if (strlen(_papi_hwi_system_info.hw_info.model_string) == 0) {
+						strSize = sizeof(_papi_hwi_system_info.hw_info.model_string);
+						strncpy( _papi_hwi_system_info.hw_info.model_string, pinfo.desc, strSize - 1);
+					}
 				}
 			}
 
@@ -1213,14 +1277,10 @@ _pe_libpfm4_init(papi_vector_t *component, int cidx,
 	}
 
 	if (!found_default) {
-		strncpy(component->cmp_info.disabled_reason,
-			"could not find default PMU",PAPI_MAX_STR_LEN);
 		return PAPI_ECMP;
 	}
 
 	if (found_default>1) {
-		strncpy(component->cmp_info.disabled_reason,
-			"found more than one default PMU",PAPI_MAX_STR_LEN);
 		return PAPI_ECOUNT;
 	}
 
@@ -1236,7 +1296,6 @@ _pe_libpfm4_init(papi_vector_t *component, int cidx,
 		retval = _papi_load_preset_table( (char *)event_table->default_pmu.name,
 				event_table->default_pmu.pmu, cidx );
 		if ( retval!=PAPI_OK ) {
-			strncpy(component->cmp_info.disabled_reason,"_papi_load_preset_table failed",PAPI_MAX_STR_LEN);
 			return PAPI_ENOEVNT;
 		}
 	}
