@@ -64,6 +64,11 @@
 #define PERF_EVENTS_OPENED  0x01
 #define PERF_EVENTS_RUNNING 0x02
 
+// The following macro follows if a string function has an error. It should 
+// never happen; but it is necessary to prevent compiler warnings. We print 
+// something just in case there is programmer error in invoking the function.
+#define HANDLE_STRING_ERROR {fprintf(stderr,"%s:%i unexpected string function error.\n",__FILE__,__LINE__); exit(-1);}
+
 /* Forward declaration */
 papi_vector_t _perf_event_vector;
 
@@ -132,7 +137,8 @@ pe_vendor_fixups(papi_vector_t *vector)
   }
 
 	/* ARM */
-	if ( _papi_hwi_system_info.hw_info.vendor == PAPI_VENDOR_ARM) {
+	/* If implementer is ARM Limited. */
+	if ( _papi_hwi_system_info.hw_info.vendor == PAPI_VENDOR_ARM_ARM) {
 
 		/* Some ARMv7 and earlier could not measure	*/
 		/* KERNEL and USER separately.			*/
@@ -268,7 +274,7 @@ get_read_format( unsigned int multiplex,
 /* attr.exclude_guest is enabled by default in recent libpfm4 */
 /* however older kernels will reject events with it set */
 /* because the reserved field is not all zeros */
-static int
+void
 check_exclude_guest( void )
 {
 	int ev_fd;
@@ -283,7 +289,7 @@ check_exclude_guest( void )
 	ev_fd = sys_perf_event_open( &attr, 0, -1, -1, 0 );
 	if ( ev_fd == -1 ) {
 		PAPIERROR("Couldn't open hw_instructions in exclude_guest=0 test");
-		return -1;
+		return;
 	}
 	close(ev_fd);
 
@@ -305,7 +311,7 @@ check_exclude_guest( void )
 		close(ev_fd);
 	}
 
-	return PAPI_OK;
+	return;
 }
 
 /*****************************************************************/
@@ -676,6 +682,12 @@ set_up_mmap( pe_control_t *ctl, int evt_idx)
 
 
 
+/* Request user access for arm64 */
+static inline void arm64_request_user_access(struct perf_event_attr *hw_event)
+{
+	hw_event->config1=0x2;        /* Request user access */
+}
+
 /* Open all events in the control state */
 static int
 open_pe_events( pe_context_t *ctx, pe_control_t *ctl )
@@ -729,6 +741,11 @@ open_pe_events( pe_context_t *ctx, pe_control_t *ctl )
 		if (( i == 0 ) || (ctl->multiplexed)) {
 			ctl->events[i].attr.pinned = !ctl->multiplexed;
 			ctl->events[i].attr.disabled = 1;
+#if defined(__aarch64__)
+			if (_perf_event_vector.cmp_info.fast_counter_read) {
+				arm64_request_user_access(&ctl->events[i].attr);
+			}
+#endif
 			ctl->events[i].group_leader_fd=-1;
 			ctl->events[i].attr.read_format = get_read_format(
 							ctl->multiplexed,
@@ -737,6 +754,11 @@ open_pe_events( pe_context_t *ctx, pe_control_t *ctl )
 		} else {
 			ctl->events[i].attr.pinned=0;
 			ctl->events[i].attr.disabled = 0;
+#if defined(__aarch64__)
+			if (_perf_event_vector.cmp_info.fast_counter_read) {
+				arm64_request_user_access(&ctl->events[i].attr);
+			}
+#endif
 			ctl->events[i].group_leader_fd=ctl->events[0].event_fd;
 			ctl->events[i].attr.read_format = get_read_format(
 							ctl->multiplexed,
@@ -1041,8 +1063,16 @@ _pe_reset( hwd_context_t *ctx, hwd_control_state_t *ctl )
 
 	/* We need to reset all of the events, not just the group leaders */
 	for( i = 0; i < pe_ctl->num_events; i++ ) {
-		ret = ioctl( pe_ctl->events[i].event_fd,
-				PERF_EVENT_IOC_RESET, NULL );
+		if (_perf_event_vector.cmp_info.fast_counter_read) {
+			ret = ioctl( pe_ctl->events[i].event_fd, 
+					PERF_EVENT_IOC_RESET, NULL );
+			pe_ctl->reset_counts[i] = mmap_read_reset_count(
+					pe_ctl->events[i].mmap_buf);
+			pe_ctl->reset_flag = 1;
+		} else {
+			ret = ioctl( pe_ctl->events[i].event_fd, 
+					PERF_EVENT_IOC_RESET, NULL );
+		}
 		if ( ret == -1 ) {
 			PAPIERROR("ioctl(%d, PERF_EVENT_IOC_RESET, NULL) "
 					"returned error, Linux says: %s",
@@ -1113,6 +1143,8 @@ _pe_rdpmc_read( hwd_context_t *ctx, hwd_control_state_t *ctl,
 	for ( i = 0; i < pe_ctl->num_events; i++ ) {
 
 		count = mmap_read_self(pe_ctl->events[i].mmap_buf,
+						pe_ctl->reset_flag,
+						pe_ctl->reset_counts[i],
 						&enabled,&running);
 
 		if (count==0xffffffffffffffffULL) {
@@ -1277,7 +1309,10 @@ _pe_read( hwd_context_t *ctx, hwd_control_state_t *ctl,
 	/* FIXME: we fallback to slow reads if *any* event in eventset fails */
 	/*        in theory we could only fall back for the one event        */
 	/*        but that makes the code more complicated.                  */
-	if ((_perf_event_vector.cmp_info.fast_counter_read) && (!pe_ctl->inherit)) {
+	if ((_perf_event_vector.cmp_info.fast_counter_read) &&
+		(!pe_ctl->inherit) &&
+		(!pe_ctl->attached) &&
+		(pe_ctl->granularity==PAPI_GRN_THR)) {
 		result=_pe_rdpmc_read( ctx, ctl, events, flags);
 		/* if successful we are done, otherwise fall back to read */
 		if (result==PAPI_OK) return PAPI_OK;
@@ -1432,6 +1467,10 @@ _pe_start( hwd_context_t *ctx, hwd_control_state_t *ctl )
 				pe_ctl->events[i].event_fd);
 			ret=ioctl( pe_ctl->events[i].event_fd,
 				PERF_EVENT_IOC_ENABLE, NULL) ;
+			if (_perf_event_vector.cmp_info.fast_counter_read) {
+				pe_ctl->reset_counts[i] = 0LL;
+				pe_ctl->reset_flag = 0;
+			}
 
 			/* ioctls always return -1 on failure */
 			if (ret == -1) {
@@ -1932,7 +1971,7 @@ _pe_dispatch_timer( int n, hwd_siginfo_t *info, void *uc)
 	( void ) n;                           /*unused */
 	_papi_hwi_context_t hw_context;
 	int found_evt_idx = -1, fd = info->si_fd;
-	caddr_t address;
+	vptr_t address;
 	ThreadInfo_t *thread = _papi_hwi_lookup_thread( 0 );
 	int i;
 	pe_control_t *ctl;
@@ -2061,7 +2100,7 @@ _pe_dispatch_timer( int n, hwd_siginfo_t *info, void *uc)
 	*/
 
 		_papi_hwi_dispatch_overflow_signal( ( void * ) &hw_context,
-					( caddr_t ) ( unsigned long ) ip,
+					( vptr_t ) ( unsigned long ) ip,
 					NULL, ( 1 << found_evt_idx ), 0,
 					&thread, cidx );
 
@@ -2291,6 +2330,29 @@ _pe_shutdown_component( void ) {
 }
 
 
+#if defined(__aarch64__)
+/* Check access PMU counter from User space for arm64 support */
+static int _pe_detect_arm64_access(void) {
+
+	FILE *fff;
+	int perf_user_access;
+	int retval;
+
+	fff=fopen("/proc/sys/kernel/perf_user_access","r");
+	if (fff==NULL) {
+		return 0;
+	}
+
+	/* 1 means you can access PMU counter from User space */
+	/* 0 means you can not access PMU counter from User space */
+	retval=fscanf(fff,"%d",&perf_user_access);
+	if (retval!=1) fprintf(stderr,"Error reading /proc/sys/kernel/perf_user_access\n");
+	fclose(fff);
+
+	return perf_user_access;
+}
+#endif
+
 /* Check the mmap page for rdpmc support */
 static int _pe_detect_rdpmc(void) {
 
@@ -2299,10 +2361,13 @@ static int _pe_detect_rdpmc(void) {
 	void *addr;
 	struct perf_event_mmap_page *our_mmap;
 	int page_size=getpagesize();
+#if defined(__aarch64__)
+	int retval;
+#endif
 
-#if defined(__i386__) || defined (__x86_64__)
+#if defined(__i386__) || defined (__x86_64__) || defined(__aarch64__)
 #else
-	/* We only support rdpmc on x86 for now */
+	/* We support rdpmc on x86 and arm64 for now */
         return 0;
 #endif
 
@@ -2312,12 +2377,23 @@ static int _pe_detect_rdpmc(void) {
 		return 0;
 	}
 
+#if defined(__aarch64__)
+	/* Detect if we can use PMU counter from User space for arm64 */
+	retval = _pe_detect_arm64_access();
+	if (retval == 0) {
+		return 0;
+	}
+#endif
+
 	/* Create a fake instructions event so we can read a mmap page */
 	memset(&pe,0,sizeof(struct perf_event_attr));
 
 	pe.type=PERF_TYPE_HARDWARE;
 	pe.size=sizeof(struct perf_event_attr);
 	pe.config=PERF_COUNT_HW_INSTRUCTIONS;
+#if defined(__aarch64__)
+	arm64_request_user_access(&pe);
+#endif
 	pe.exclude_kernel=1;
 	pe.disabled=1;
 
@@ -2372,6 +2448,7 @@ _pe_handle_paranoid(papi_vector_t *component) {
 	FILE *fff;
 	int paranoid_level;
 	int retval;
+   char *strCpy;
 
 	/* The is the official way to detect if perf_event support exists */
 	/* The file is called perf_counter_paranoid on 2.6.31             */
@@ -2379,9 +2456,10 @@ _pe_handle_paranoid(papi_vector_t *component) {
 
 	fff=fopen("/proc/sys/kernel/perf_event_paranoid","r");
 	if (fff==NULL) {
-		strncpy(component->cmp_info.disabled_reason,
+		strCpy=strncpy(component->cmp_info.disabled_reason,
 			"perf_event support not detected",PAPI_MAX_STR_LEN);
-		return PAPI_ENOCMP;
+      if (strCpy == NULL) HANDLE_STRING_ERROR;
+		return PAPI_ECMP;
 	}
 
 	/* 3 (vendor patch) means completely disabled */
@@ -2394,9 +2472,10 @@ _pe_handle_paranoid(papi_vector_t *component) {
 	fclose(fff);
 
 	if (paranoid_level==3) {
-		strncpy(component->cmp_info.disabled_reason,
+		strCpy=strncpy(component->cmp_info.disabled_reason,
 			"perf_event support disabled by Linux with paranoid=3",PAPI_MAX_STR_LEN);
-		return PAPI_ENOCMP;
+      if (strCpy == NULL) HANDLE_STRING_ERROR;
+		return PAPI_ECMP;
 	}
 
 	if ((paranoid_level==2) && (getuid()!=0)) {
@@ -2458,12 +2537,14 @@ _pe_init_component( int cidx )
 {
 
 	int retval;
+   char *strCpy;
 
 	our_cidx=cidx;
 
 	/* Update component behavior based on paranoid setting */
 	retval=_pe_handle_paranoid(_papi_hwd[cidx]);
-	if (retval!=PAPI_OK) return retval;
+   
+	if (retval!=PAPI_OK) goto fn_fail; // disabled_reason handled by _pe_handle_paranoid.
 
 #if (OBSOLETE_WORKAROUNDS==1)
 	/* Handle any kernel version related workarounds */
@@ -2473,9 +2554,10 @@ _pe_init_component( int cidx )
 	/* Setup mmtimers, if appropriate */
 	retval=mmtimer_setup();
 	if (retval) {
-		strncpy(_papi_hwd[cidx]->cmp_info.disabled_reason,
+		strCpy=strncpy(_papi_hwd[cidx]->cmp_info.disabled_reason,
 			"Error initializing mmtimer",PAPI_MAX_STR_LEN);
-		return retval;
+      if (strCpy == NULL) HANDLE_STRING_ERROR;
+		goto fn_fail;
 	}
 
 	/* Set the overflow signal */
@@ -2504,10 +2586,10 @@ _pe_init_component( int cidx )
 	/* Run the libpfm4-specific setup */
 	retval = _papi_libpfm4_init(_papi_hwd[cidx]);
 	if (retval) {
-
-		strncpy(_papi_hwd[cidx]->cmp_info.disabled_reason,
+		strCpy=strncpy(_papi_hwd[cidx]->cmp_info.disabled_reason,
 			"Error initializing libpfm4",PAPI_MAX_STR_LEN);
-		return retval;
+      if (strCpy == NULL) HANDLE_STRING_ERROR;
+		goto fn_fail;
 
 	}
 
@@ -2520,37 +2602,43 @@ _pe_init_component( int cidx )
 	if (retval) {
 		switch(retval) {
 			case PAPI_ENOMEM:
-				strncpy(_papi_hwd[cidx]->cmp_info.disabled_reason,
+				strCpy=strncpy(_papi_hwd[cidx]->cmp_info.disabled_reason,
 					"Error libpfm4 memory allocation",
 					PAPI_MAX_STR_LEN);
+            if (strCpy == NULL) HANDLE_STRING_ERROR;
 				break;
 			case PAPI_ENOSUPP:
-				strncpy(_papi_hwd[cidx]->cmp_info.disabled_reason,
+				strCpy=strncpy(_papi_hwd[cidx]->cmp_info.disabled_reason,
 					"Error libpfm4 no PMUs found",
 					PAPI_MAX_STR_LEN);
+            if (strCpy == NULL) HANDLE_STRING_ERROR;
 				break;
-			case PAPI_ENOCMP:
-				strncpy(_papi_hwd[cidx]->cmp_info.disabled_reason,
+			case PAPI_ECMP:
+				strCpy=strncpy(_papi_hwd[cidx]->cmp_info.disabled_reason,
 					"Error libpfm4 no default PMU found",
 					PAPI_MAX_STR_LEN);
+            if (strCpy == NULL) HANDLE_STRING_ERROR;
 				break;
 			case PAPI_ECOUNT:
-				strncpy(_papi_hwd[cidx]->cmp_info.disabled_reason,
+				strCpy=strncpy(_papi_hwd[cidx]->cmp_info.disabled_reason,
 					"Error libpfm4 too many default PMUs found",
 					PAPI_MAX_STR_LEN);
+            if (strCpy == NULL) HANDLE_STRING_ERROR;
 				break;
 			case PAPI_ENOEVNT:
-				strncpy(_papi_hwd[cidx]->cmp_info.disabled_reason,
+				strCpy=strncpy(_papi_hwd[cidx]->cmp_info.disabled_reason,
 					"Error loading preset events",
 					PAPI_MAX_STR_LEN);
+            if (strCpy == NULL) HANDLE_STRING_ERROR;
 				break;
 			default:
-				strncpy(_papi_hwd[cidx]->cmp_info.disabled_reason,
+				strCpy=strncpy(_papi_hwd[cidx]->cmp_info.disabled_reason,
 					"Unknown libpfm4 related error",
 					PAPI_MAX_STR_LEN);
+            if (strCpy == NULL) HANDLE_STRING_ERROR;
 
 		}
-		return retval;
+        goto fn_fail;
 	}
 
 	/* Detect NMI watchdog which can steal counters */
@@ -2566,7 +2654,11 @@ _pe_init_component( int cidx )
 	/* check for exclude_guest issue */
 	check_exclude_guest();
 
-	return PAPI_OK;
+  fn_exit:
+    _papi_hwd[cidx]->cmp_info.disabled = retval;
+    return retval;
+  fn_fail:
+    goto fn_exit;
 
 }
 
