@@ -10,13 +10,15 @@
 #include "papi.h"
 #include "icache.h"
 
-void i_cache_driver(char* papi_event_name, int init, char* outdir, int show_progress)
+void i_cache_driver(char* papi_event_name, int junk, hw_desc_t *hw_desc, char* outdir, int show_progress)
 {
     // Open output file.
-    const char *sufx = ".instr";
+    const char *sufx = ".icache";
     char *papiFileName;
     FILE *ofp_papi;
 
+    (void)hw_desc;
+
     int l = strlen(outdir)+strlen(papi_event_name)+strlen(sufx);
     if (NULL == (papiFileName = (char *)calloc( 1+l, sizeof(char) ))) {
         fprintf(stderr, "Failed to allocate papiFileName.\n");
@@ -33,7 +35,7 @@ void i_cache_driver(char* papi_event_name, int init, char* outdir, int show_prog
         return;
     }
 
-    seq_driver(ofp_papi, papi_event_name, init, show_progress);
+    seq_driver(ofp_papi, papi_event_name, junk, show_progress);
 
     // Close output file.
     fclose(ofp_papi);
