@@ -1,5 +1,6 @@
 #include <stdlib.h>
 #include <string.h>
+#include <strings.h>
 #include <inttypes.h>
 #include <unistd.h>
 #include <fcntl.h>
@@ -8,15 +9,25 @@
 #include "papi.h"
 #include "driver.h"
 
+#if defined(USE_MPI)
+#include <mpi.h>
+#endif
+
 int main(int argc, char*argv[])
 {
     int cmbtotal = 0, ct = 0, track = 0, ret = 0;
-    int bench_type = 0;
-    int mode = 0, pk = 0, max_iter = 1, i = 0, nevts = 0, show_progress = 0, status;
+    int i, nevts = 0, status;
     int *cards = NULL, *indexmemo = NULL;
-    char *infile = NULL, *outdir = NULL;
     char **allevts = NULL, **basenames = NULL;
     evstock *data = NULL;
+    cat_params_t params = {-1,0,1,0,0,0,NULL,NULL,NULL};
+    int nprocs = 1, myid = 0;
+
+#if defined(USE_MPI)
+    MPI_Init(&argc, &argv);
+    MPI_Comm_size(MPI_COMM_WORLD, &nprocs);
+    MPI_Comm_rank(MPI_COMM_WORLD, &myid);
+#endif
 
     // Initialize PAPI.
     ret = PAPI_library_init(PAPI_VER_CURRENT);
@@ -26,11 +37,19 @@ int main(int argc, char*argv[])
         return 0;
     }
 
+    // Initialize PAPI thread support.
+    ret = PAPI_thread_init( omp_get_thread_num_wrapper );
+    if( ret != PAPI_OK ) {
+
+        fprintf(stderr,"PAPI thread init error: %s Exiting...\n", PAPI_strerror(ret));
+        return 0;
+    }
+
     // Parse the command-line arguments.
-    status = parseArgs(argc, argv, &pk, &mode, &max_iter, &infile, &outdir, &bench_type, &show_progress );
+    status = parseArgs(argc, argv, &params);
     if(0 != status)
     {
-        free(outdir);
+        free(params.outputdir);
         PAPI_shutdown();
         return 0;
     }
@@ -39,19 +58,19 @@ int main(int argc, char*argv[])
     data = (evstock*)calloc(1,sizeof(evstock));
     if(NULL == data)
     {
-        free(outdir);
+        free(params.outputdir);
         fprintf(stderr, "Could not initialize event stock. Exiting...\n");
         PAPI_shutdown();
         return 0;
     }
 
     // Read the list of base event names and maximum qualifier set cardinalities.
-    if( READ_FROM_FILE == mode)
+    if( READ_FROM_FILE == params.mode)
     {
-        ct = setup_evts(infile, &basenames, &cards);
+        ct = setup_evts(params.inputfile, &basenames, &cards);
         if(ct == -1)
         {
-            free(outdir);
+            free(params.outputdir);
             remove_stock(data);
             PAPI_shutdown();
             return 0;
@@ -62,9 +81,9 @@ int main(int argc, char*argv[])
     status = build_stock(data);
     if(status)
     {
-        free(outdir);
+        free(params.outputdir);
         remove_stock(data);
-        if(READ_FROM_FILE == mode)
+        if(READ_FROM_FILE == params.mode)
         {
             for(i = 0; i < ct; ++i)
             {
@@ -82,12 +101,12 @@ int main(int argc, char*argv[])
     nevts = num_evts(data);
 
     // Verify the validity of the cardinalities.
-    cmbtotal = check_cards(mode, &indexmemo, basenames, cards, ct, nevts, pk, data);
+    cmbtotal = check_cards(params, &indexmemo, basenames, cards, ct, nevts, data);
     if(-1 == cmbtotal)
     {
-        free(outdir);
+        free(params.outputdir);
         remove_stock(data);
-        if(READ_FROM_FILE == mode)
+        if(READ_FROM_FILE == params.mode)
         {
             for(i = 0; i < ct; ++i)
             {
@@ -108,15 +127,41 @@ int main(int argc, char*argv[])
     }
 
     // Create the qualifier combinations for each event.
-    trav_evts(data, pk, cards, nevts, ct, mode, allevts, &track, indexmemo, basenames);
+    trav_evts(data, params.subsetsize, cards, nevts, ct, params.mode, allevts, &track, indexmemo, basenames);
+
+    char *conf_file_name = ".cat_cfg";
+    if( NULL != params.conf_file ) {
+        conf_file_name = params.conf_file;
+    }
+    hw_desc_t *hw_desc = obtain_hardware_description(conf_file_name);
+
+    /* Set the default number of threads to the OMP_NUM_THREADS environment
+     * variable if it is defined. Otherwise, set it to the number of CPUs
+     * in a single socket. */
+    int numSetThreads = 1;
+    char* envVarDefined = getenv("OMP_NUM_THREADS");
+    if (NULL == envVarDefined) {
+        omp_set_num_threads(hw_desc->numcpus);
+
+        #pragma omp parallel default(shared)
+        {
+            if(!omp_get_thread_num()) {
+                numSetThreads = omp_get_num_threads();
+            }
+        }
+
+        if (numSetThreads != hw_desc->numcpus) {
+            fprintf(stderr, "Warning! Failed to set default number of threads to number of CPUs in a single socket.\n");
+        }
+    }
 
     // Run the benchmark for each qualifier combination.
-    testbench(allevts, cmbtotal, max_iter, argc, outdir, bench_type, show_progress);
+    testbench(allevts, cmbtotal, hw_desc, params, myid, nprocs);
 
     // Free dynamically allocated memory.
-    free(outdir);
+    free(params.outputdir);
     remove_stock(data);
-    if(READ_FROM_FILE == mode)
+    if(READ_FROM_FILE == params.mode)
     {
         for(i = 0; i < ct; ++i)
         {
@@ -131,22 +176,38 @@ int main(int argc, char*argv[])
         free(allevts[i]);
     }
     free(allevts);
+    free(hw_desc);
 
     PAPI_shutdown();
+
+#if defined(USE_MPI)
+    MPI_Barrier(MPI_COMM_WORLD);
+    MPI_Finalize();
+#endif
+
     return 0;
 }
 
+unsigned long int omp_get_thread_num_wrapper() {
+    return omp_get_thread_num();
+}
+
 // Verify that valid qualifier counts are provided and count their combinations.
-int check_cards(int mode, int** indexmemo, char** basenames, int* cards, int ct, int nevts, int pk, evstock* data)
+int check_cards(cat_params_t params, int** indexmemo, char** basenames, int* cards, int ct, int nevts, evstock* data)
 {
     int i, j, minim, n, cmbtotal = 0;
     char *name;
+    int mode = params.mode;
+    int pk = params.subsetsize;
 
     // User provided a file of events.
     if(READ_FROM_FILE == mode)
     {
         // Compute the total number of qualifier combinations and allocate memory to store them.
-        (*indexmemo) = (int*)malloc(ct*sizeof(int));
+        if (NULL == ((*indexmemo) = (int*)malloc(ct*sizeof(int)))) {
+            fprintf(stderr, "Failed to allocate memory.\n");
+            return 0;
+        }
 
         // Find the index in the main stock whose event corresponds to that in the file provided.
         // This simplifies looking up event qualifiers later.
@@ -243,18 +304,215 @@ int check_cards(int mode, int** indexmemo, char** basenames, int* cards, int ct,
     return cmbtotal;
 }
 
+static hw_desc_t *obtain_hardware_description(char *conf_file_name){
+    int i,j;
+    hw_desc_t *hw_desc;
+	PAPI_mh_level_t *L;
+    const PAPI_hw_info_t *meminfo;
+
+    // Allocate some space.
+    hw_desc = (hw_desc_t *)calloc(1, sizeof(hw_desc_t));
+
+    // Set at least the L1 cache size to a default value.
+    hw_desc->dcache_line_size[0] = 64;
+
+    // Set other default values.
+    for( i=0; i<_MAX_SUPPORTED_CACHE_LEVELS; ++i ) {
+        hw_desc->split[i] = 1;
+        hw_desc->pts_per_reg[i] = 3;
+    }
+    hw_desc->mmsplit = 1;
+    hw_desc->pts_per_mm = 3;
+    hw_desc->maxPPB = 512;
+
+    // Obtain hardware values through PAPI_get_hardware_info().
+    meminfo = PAPI_get_hardware_info();
+    if( NULL != meminfo ) {
+        hw_desc->numcpus = meminfo->ncpu;
+        hw_desc->cache_levels = meminfo->mem_hierarchy.levels;
+	    L = ( PAPI_mh_level_t * ) & ( meminfo->mem_hierarchy.level[0] );
+        for ( i = 0; i < meminfo->mem_hierarchy.levels && i<_MAX_SUPPORTED_CACHE_LEVELS; i++ ) {
+            for ( j = 0; j < 2; j++ ) {
+                if ( (PAPI_MH_TYPE_DATA == PAPI_MH_CACHE_TYPE(L[i].cache[j].type)) ||
+                     (PAPI_MH_TYPE_UNIFIED == PAPI_MH_CACHE_TYPE(L[i].cache[j].type)) ){
+                    hw_desc->dcache_line_size[i] = L[i].cache[j].line_size;
+                    hw_desc->dcache_size[i] =      L[i].cache[j].size;
+                    hw_desc->dcache_assoc[i] =     L[i].cache[j].associativity;
+                }
+                if ( (PAPI_MH_TYPE_INST == PAPI_MH_CACHE_TYPE(L[i].cache[j].type)) ||
+                     (PAPI_MH_TYPE_UNIFIED == PAPI_MH_CACHE_TYPE(L[i].cache[j].type)) ){
+                    hw_desc->icache_line_size[i] = L[i].cache[j].line_size;
+                    hw_desc->icache_size[i] =      L[i].cache[j].size;
+                    hw_desc->icache_assoc[i] =     L[i].cache[j].associativity;
+                }
+            }
+        }
+    }
+
+    // Read the config file, if there, in case the user wants to overwrite some values.
+    read_conf_file(conf_file_name, hw_desc);
+    return hw_desc;
+}
+
+
+
+static int parse_line(FILE *input, char **key, long long *value){
+    int status;
+    size_t linelen=0, len;
+    char *line=NULL;
+    char *pos=NULL;
+
+    // Read one line from the input file.
+    int ret_val = (int)getline(&line, &linelen, input);
+    if( ret_val < 0 )
+        return ret_val;
+
+    // Kill the part of the line after the comment character '#'.
+    pos = strchr(line, '#');
+    if( NULL != pos ){
+        *pos = '\0';
+    }
+
+    // Make sure the line is an assignment.
+    pos = strchr(line, '=');
+    if( NULL == pos ){
+        goto handle_error;
+    }
+
+    len = strcspn(line, " =");
+    *key = (char *)calloc((1+len),sizeof(char));
+    strncpy(*key, line, len);
+
+    // Scan the line to make sure it has the form "key = value"
+    status = sscanf(pos, "= %lld", value);
+    if(1 != status){
+        fprintf(stderr,"Malformed line in conf file: '%s'\n", line);
+        goto handle_error;
+    }
+
+    return 0;
+
+handle_error:
+    free(line);
+    key = NULL;
+    *value = 0;
+    line = NULL;
+    linelen = 0;
+    return 1;
+}
+
+
+static void read_conf_file(char *conf_file_name, hw_desc_t *hw_desc){
+    FILE *input;
+
+    // Try to open the file.
+    input = fopen(conf_file_name, "r");
+    if (NULL == input ){
+        return;
+    }
+
+    while(1){
+        long long value;
+        char *key=NULL;
+
+        int ret_val = parse_line(input, &key, &value);
+        if( ret_val < 0 ){
+            free(key);
+            break;
+        }else if( ret_val > 0 ){
+            continue;
+        }
+
+        // If the user has set "AUTO_DISCOVERY_MODE = 1" then we don't need to process this file.
+        // Otherwise, any entry in this file should overwrite what we auto discovered.
+        if( !strcmp(key, "AUTO_DISCOVERY_MODE") && (value == 1) ){
+            return;
+        // Data caches (including unified caches)
+        }else if( !strcmp(key, "L1_DCACHE_LINE_SIZE") || !strcmp(key, "L1_UCACHE_LINE_SIZE") ){
+            hw_desc->dcache_line_size[0] = value;
+        }else if( !strcmp(key, "L2_DCACHE_LINE_SIZE") || !strcmp(key, "L2_UCACHE_LINE_SIZE") ){
+            hw_desc->dcache_line_size[1] = value;
+        }else if( !strcmp(key, "L3_DCACHE_LINE_SIZE") || !strcmp(key, "L3_UCACHE_LINE_SIZE") ){
+            hw_desc->dcache_line_size[2] = value;
+        }else if( !strcmp(key, "L4_DCACHE_LINE_SIZE") || !strcmp(key, "L4_UCACHE_LINE_SIZE") ){
+            hw_desc->dcache_line_size[3] = value;
+        }else if( !strcmp(key, "L1_DCACHE_SIZE") || !strcmp(key, "L1_UCACHE_SIZE") ){
+            if( hw_desc->cache_levels < 1 ) hw_desc->cache_levels = 1;
+            hw_desc->dcache_size[0] = value;
+        }else if( !strcmp(key, "L2_DCACHE_SIZE") || !strcmp(key, "L2_UCACHE_SIZE") ){
+            if( hw_desc->cache_levels < 2 ) hw_desc->cache_levels = 2;
+            hw_desc->dcache_size[1] = value;
+        }else if( !strcmp(key, "L3_DCACHE_SIZE") || !strcmp(key, "L3_UCACHE_SIZE") ){
+            if( hw_desc->cache_levels < 3 ) hw_desc->cache_levels = 3;
+            hw_desc->dcache_size[2] = value;
+        }else if( !strcmp(key, "L4_DCACHE_SIZE") || !strcmp(key, "L4_UCACHE_SIZE") ){
+            if( hw_desc->cache_levels < 4 ) hw_desc->cache_levels = 4;
+            hw_desc->dcache_size[3] = value;
+        // Instruction caches (including unified caches)
+        }else if( !strcmp(key, "L1_ICACHE_LINE_SIZE") || !strcmp(key, "L1_UCACHE_LINE_SIZE") ){
+            hw_desc->icache_line_size[0] = value;
+        }else if( !strcmp(key, "L2_ICACHE_LINE_SIZE") || !strcmp(key, "L2_UCACHE_LINE_SIZE") ){
+            hw_desc->icache_line_size[1] = value;
+        }else if( !strcmp(key, "L3_ICACHE_LINE_SIZE") || !strcmp(key, "L3_UCACHE_LINE_SIZE") ){
+            hw_desc->icache_line_size[2] = value;
+        }else if( !strcmp(key, "L4_ICACHE_LINE_SIZE") || !strcmp(key, "L4_UCACHE_LINE_SIZE") ){
+            hw_desc->icache_line_size[3] = value;
+        }else if( !strcmp(key, "L1_ICACHE_SIZE") || !strcmp(key, "L1_UCACHE_SIZE") ){
+            hw_desc->icache_size[0] = value;
+        }else if( !strcmp(key, "L2_ICACHE_SIZE") || !strcmp(key, "L2_UCACHE_SIZE") ){
+            hw_desc->icache_size[1] = value;
+        }else if( !strcmp(key, "L3_ICACHE_SIZE") || !strcmp(key, "L3_UCACHE_SIZE") ){
+            hw_desc->icache_size[2] = value;
+        }else if( !strcmp(key, "L4_ICACHE_SIZE") || !strcmp(key, "L4_UCACHE_SIZE") ){
+            hw_desc->icache_size[3] = value;
+        }else if( !strcmp(key, "L1_SPLIT") ){
+            hw_desc->split[0] = value;
+        }else if( !strcmp(key, "L2_SPLIT") ){
+            hw_desc->split[1] = value;
+        }else if( !strcmp(key, "L3_SPLIT") ){
+            hw_desc->split[2] = value;
+        }else if( !strcmp(key, "L4_SPLIT") ){
+            hw_desc->split[3] = value;
+        }else if( !strcmp(key, "MM_SPLIT") ){
+            hw_desc->mmsplit = value;
+        }else if( !strcmp(key, "PTS_PER_L1") ){
+            hw_desc->pts_per_reg[0] = value;
+        }else if( !strcmp(key, "PTS_PER_L2") ){
+            hw_desc->pts_per_reg[1] = value;
+        }else if( !strcmp(key, "PTS_PER_L3") ){
+            hw_desc->pts_per_reg[2] = value;
+        }else if( !strcmp(key, "PTS_PER_L4") ){
+            hw_desc->pts_per_reg[3] = value;
+        }else if( !strcmp(key, "PTS_PER_MM") ){
+            hw_desc->pts_per_mm = value;
+        }else if( !strcmp(key, "MAX_PPB") ){
+            hw_desc->maxPPB = value;
+        }
+
+        free(key);
+        key = NULL;
+    }
+    fclose(input);
+    return;
+}
+
 // Read the contents of the file supplied by the user.
 int setup_evts(char* inputfile, char*** basenames, int** evnt_cards)
 {
     size_t linelen = 0;
     int cnt = 0, status = 0;
-    char *line, *place;
+    char *line = NULL, *place;
     FILE *input;
     int evnt_count = 256;
   
     char **names = (char **)calloc(evnt_count, sizeof(char *));
     int *cards = (int *)calloc(evnt_count, sizeof(int));
 
+    if (NULL == names || NULL == cards) {
+        fprintf(stderr, "Failed to allocate memory.\n");
+        return 0;
+    }
+
     // Read the base event name and cardinality columns.
     input = fopen(inputfile, "r");
     for(cnt=0; 1; cnt++)
@@ -266,11 +524,27 @@ int setup_evts(char* inputfile, char*** basenames, int** evnt_cards)
         {
             evnt_count *= 2;
             names = realloc(names, evnt_count*sizeof(char *));
+            cards = realloc(cards, evnt_count*sizeof(int));
+
+            if (NULL == names || NULL == cards) {
+                fprintf(stderr, "Failed to allocate memory.\n");
+                return 0;
+            }
         }
 
         place = strstr(line, " ");
-        if( NULL == place )
-        {
+
+        // If this line was commented, silently ignore it.
+        if(strlen(line) > 0 && line[0] == '#') {
+            names[cnt] = NULL;
+            cards[cnt] = -1;
+            cnt--;
+
+            free(line);
+            line = NULL;
+            linelen = 0;
+            continue;
+        } else if( NULL == place ) {
             fprintf(stderr,"problem with line: '%s'\n",line);
             names[cnt] = NULL;
             cards[cnt] = -1;
@@ -285,7 +559,7 @@ int setup_evts(char* inputfile, char*** basenames, int** evnt_cards)
         names[cnt] = NULL;
         status = sscanf(line, "%ms %d", &(names[cnt]), &(cards[cnt]) );
 
-        // If this line was malformed, silently ignore it.
+        // If this line was malformed, ignore it.
         if(2 != status)
         {
             fprintf(stderr,"problem with line: '%s'\n",line);
@@ -355,7 +629,11 @@ void combine_qualifiers(int n, int pk, int ct, char** list, char* name, char** a
                 }
             }
 
-            chunk = (char*)malloc((evtsize+1)*sizeof(char));
+            if (NULL == (chunk = (char*)malloc((evtsize+1)*sizeof(char)))) {
+                fprintf(stderr, "Failed to allocate memory.\n");
+                return;
+            }
+
             strcpy(chunk,name);
             for(i = 0; i < n; ++i)
             {
@@ -418,6 +696,11 @@ void trav_evts(evstock* stock, int pk, int* cards, int nevts, int selexnsize, in
             {
                 chosen = (char**)malloc(n*sizeof(char*));
                 bitmap = (int*)calloc(n, sizeof(int));    
+                
+                if (NULL == chosen || NULL == bitmap) {
+                    fprintf(stderr, "Failed to allocate memory.\n");
+                    return;
+                }
 
                 // Store the qualifiers for the current event.
                 for(k = 0; k < n; ++k)
@@ -466,6 +749,11 @@ void trav_evts(evstock* stock, int pk, int* cards, int nevts, int selexnsize, in
             chosen = (char**)malloc(n*sizeof(char*));
             bitmap = (int*)calloc(n, sizeof(int));    
 
+            if (NULL == chosen || NULL == bitmap) {
+                fprintf(stderr, "Failed to allocate memory.\n");
+                return;
+            }
+
             // Store the qualifiers for the current event.
             for(j = 0; j < n; ++j)
             {
@@ -518,34 +806,6 @@ int comb(int n, int k)
     return perm(n, k)/perm(k, k);
 }
 
-// Measures the read latencies of the data cache. This information is
-// useful for analyzing data cache-related event signatures.
-void get_dcache_latencies(int max_iter, char *outputdir){
-    FILE *ofp;
-
-    // Make sure the output files could be opened.
-    int l = strlen(outputdir)+strlen("latencies.txt");
-    char *latencyFileName = (char *)calloc( 1+l, sizeof(char) );
-    if (!latencyFileName) {
-        fprintf(stderr, "Unable to allocate memory. Skipping latency test.\n");
-        return;
-    }
-    if (l != (sprintf(latencyFileName, "%slatencies.txt", outputdir))) {
-        fprintf(stderr, "sprintf error.\n");
-        return;
-    }
-    if (NULL == (ofp = fopen(latencyFileName,"w"))) {
-        fprintf(stderr, "Unable to open file %s. Skipping latency test.\n", latencyFileName);
-        return;
-    }
-
-    d_cache_test(3, max_iter, 256, 128, NULL, 1, 0, ofp);
-
-    fclose(ofp);
-
-    return;
-}
-
 static void print_progress(int prg)
 {
     if(prg < 100)
@@ -566,9 +826,19 @@ static void print_progress2(int prg)
     fflush(stdout);
 }
 
-void testbench(char** allevts, int cmbtotal, int max_iter, int init, char* outputdir, int bench_type, int show_progress )
+void testbench(char** allevts, int cmbtotal, hw_desc_t *hw_desc, cat_params_t params, int myid, int nprocs)
 {
     int i;
+    int junk=((int)getpid()+123)/456;
+    int low = myid*(cmbtotal/nprocs);
+    int cap = (myid+1)*(cmbtotal/nprocs);
+    int offset = nprocs*(1+cmbtotal/nprocs)-cmbtotal;
+
+    // Divide the work as evenly as possible.
+    if(myid >= offset) {
+        cap += myid-offset+1;
+        low += myid-offset;
+    }
 
     // Make sure the user provided events and iterate through all events.
     if( 0 == cmbtotal )
@@ -578,112 +848,144 @@ void testbench(char** allevts, int cmbtotal, int max_iter, int init, char* outpu
     }
 
     // Run the branch benchmark by default if none are specified.
-    if( 0 == bench_type )
+    if( 0 == params.bench_type )
     {
-        bench_type |= BENCH_BRANCH;
+        params.bench_type |= BENCH_BRANCH;
         fprintf(stderr, "Warning: No benchmark specified. Running 'branch' by default.\n");
     }
 
     /* Benchmark I - Branch*/
-    if( bench_type & BENCH_BRANCH )
+    if( params.bench_type & BENCH_BRANCH )
     {
-        if(show_progress) printf("Branch Benchmarks: ");
+        if(params.show_progress) printf("Branch Benchmarks: ");
 
-        for(i = 0; i < cmbtotal; ++i)
+        for(i = low; i < cap; ++i)
         {
-            if(show_progress) print_progress((100*i)/cmbtotal);
+            if(params.show_progress) print_progress((100*i)/cmbtotal);
 
             if( allevts[i] != NULL )
-                branch_driver(allevts[i], init, outputdir);
+                branch_driver(allevts[i], junk, hw_desc, params.outputdir);
         }
-        if(show_progress) print_progress(100);
+        if(params.show_progress) print_progress(100);
     }
 
     /* Benchmark II - Data Cache Reads*/
-    if( bench_type & BENCH_DCACHE_READ )
+    if( params.bench_type & BENCH_DCACHE_READ )
     {
-        if(show_progress)
+        if ( !params.quick && 0 == myid )
         {
-            printf("D-Cache Latencies: ");
-            fflush(stdout);
+            if(params.show_progress)
+            {
+                printf("D-Cache Latencies:  0%%\b\b\b\b");
+                fflush(stdout);
+            }
+            d_cache_driver("cat::latencies", params, hw_desc, 1, 0);
+            if(params.show_progress) printf("100%%\n");
         }
-        get_dcache_latencies(max_iter, outputdir);
-        if(show_progress) printf("100%%\n");
 
-        if(show_progress) printf("D-Cache Read Benchmarks: ");
-        for(i = 0; i < cmbtotal; ++i)
+        if(params.show_progress) printf("D-Cache Read Benchmarks: ");
+        for(i = low; i < cap; ++i)
         {
-            if(show_progress) print_progress2((100*i)/cmbtotal);
+            if(params.show_progress) print_progress2((100*i)/cmbtotal);
 
             if( allevts[i] != NULL ) {
-                d_cache_driver(allevts[i], max_iter, outputdir, 0, 0, show_progress);
+                d_cache_driver(allevts[i], params, hw_desc, 0, 0);
             }
         }
-        if(show_progress) print_progress2(100);
+        if(params.show_progress) print_progress2(100);
     }
 
     /* Benchmark III - Data Cache Writes*/
-    if( bench_type & BENCH_DCACHE_WRITE )
+    if( params.bench_type & BENCH_DCACHE_WRITE )
     {
         // If the READ benchmark was run, do not recompute the latencies.
-        if ( !(bench_type & BENCH_DCACHE_READ) )
+        if ( !(params.bench_type & BENCH_DCACHE_READ) && !params.quick)
         {
-            if(show_progress)
+            if(params.show_progress)
             {
-                printf("D-Cache Latencies: ");
+                printf("D-Cache Latencies:  0%%\b\b\b\b");
                 fflush(stdout);
             }
-            get_dcache_latencies(max_iter, outputdir);
-            if(show_progress) printf("100%%\n");
+            d_cache_driver("cat::latencies", params, hw_desc, 1, 0);
+            if(params.show_progress) printf("100%%\n");
         }
 
-        if(show_progress) printf("D-Cache Write Benchmarks: ");
-        for(i = 0; i < cmbtotal; ++i)
+        if(params.show_progress) printf("D-Cache Write Benchmarks: ");
+        for(i = low; i < cap; ++i)
         {
-            if(show_progress) print_progress2((100*i)/cmbtotal);
+            if(params.show_progress) print_progress2((100*i)/cmbtotal);
 
             if( allevts[i] != NULL ) {
-                d_cache_driver(allevts[i], max_iter, outputdir, 0, 1, show_progress);
+                d_cache_driver(allevts[i], params, hw_desc, 0, 1);
             }
         }
-        if(show_progress) print_progress2(100);
+        if(params.show_progress) print_progress2(100);
     }
 
     /* Benchmark IV - FLOPS*/
-    if( bench_type & BENCH_FLOPS )
+    if( params.bench_type & BENCH_FLOPS )
     {
-        if(show_progress) printf("FLOP Benchmarks: ");
+        if(params.show_progress) printf("FLOP Benchmarks: ");
 
-        for(i = 0; i < cmbtotal; ++i)
+        for(i = low; i < cap; ++i)
         {
-            if(show_progress) print_progress((100*i)/cmbtotal);
+            if(params.show_progress) print_progress((100*i)/cmbtotal);
 
             if( allevts[i] != NULL )
-                flops_driver(allevts[i], outputdir);
+                flops_driver(allevts[i], hw_desc, params.outputdir);
         }
-        if(show_progress) print_progress(100);
+        if(params.show_progress) print_progress(100);
     }
 
     /* Benchmark V - Instruction Cache*/
-    if( bench_type & BENCH_ICACHE_READ )
+    if( params.bench_type & BENCH_ICACHE_READ )
     {
-        if(show_progress) printf("I-Cache Benchmarks: ");
+        if(params.show_progress) printf("I-Cache Benchmarks: ");
 
-        for(i = 0; i < cmbtotal; ++i)
+        for(i = low; i < cap; ++i)
         {
-            if(show_progress) print_progress2((100*i)/cmbtotal);
+            if(params.show_progress) print_progress2((100*i)/cmbtotal);
 
             if( allevts[i] != NULL )
-                i_cache_driver(allevts[i], init, outputdir, show_progress);
+                i_cache_driver(allevts[i], junk, hw_desc, params.outputdir, params.show_progress);
         }
-        if(show_progress) print_progress2(100);
+        if(params.show_progress) print_progress2(100);
+    }
+
+    /* Benchmark VI - Vector FLOPS*/
+    if( params.bench_type & BENCH_VEC )
+    {
+        if(params.show_progress) printf("Vector FLOP Benchmarks: ");
+
+        for(i = low; i < cap; ++i)
+        {
+            if(params.show_progress) print_progress((100*i)/cmbtotal);
+
+            if( allevts[i] != NULL )
+                vec_driver(allevts[i], hw_desc, params.outputdir);
+        }
+        if(params.show_progress) print_progress(100);
+    }
+
+    /* Benchmark VII - Instructions*/
+    if( params.bench_type & BENCH_INSTR )
+    {
+        if(params.show_progress) printf("Instruction Benchmarks: ");
+
+        for(i = low; i < cap; ++i)
+        {
+            if(params.show_progress) print_progress((100*i)/cmbtotal);
+
+            if( allevts[i] != NULL )
+                instr_driver(allevts[i], hw_desc, params.outputdir);
+        }
+        if(params.show_progress) print_progress(100);
     }
 
     return;
 }
 
-int parseArgs(int argc, char **argv, int *subsetsize, int *mode, int *numit, char **inputfile, char **outputdir, int *bench_type, int *show_progress){
-
+int parseArgs(int argc, char **argv, cat_params_t *params){
     char *name = argv[0];
     char *tmp = NULL;
     int dirlen = 0;
@@ -692,8 +994,7 @@ int parseArgs(int argc, char **argv, int *subsetsize, int *mode, int *numit, cha
     FILE *test = NULL;
     int len, status = 0;
 
-    *subsetsize = -1;
-    *show_progress=0;
+    params->subsetsize = -1;
 
     // Parse the command line arguments
     while(--argc){
@@ -703,27 +1004,33 @@ int parseArgs(int argc, char **argv, int *subsetsize, int *mode, int *numit, cha
             return -1;
         }
         if( argc > 1 && !strcmp(argv[0],"-k") ){
-            *subsetsize = atoi(argv[1]);
-            if( *subsetsize < 0 )
+            params->subsetsize = atoi(argv[1]);
+            if( params->subsetsize < 0 )
             {
-                *subsetsize = 0;
+                params->subsetsize = 0;
                 fprintf(stderr, "Warning: Cannot pass a negative value to -k.\n");
             }
-            *mode = USE_ALL_EVENTS;
+            params->mode = USE_ALL_EVENTS;
             kflag = 1;
             --argc;
             ++argv;
             continue;
         }
         if( argc > 1 && !strcmp(argv[0],"-n") ){
-            *numit = atoi(argv[1]);
+            params->max_iter = atoi(argv[1]);
+            --argc;
+            ++argv;
+            continue;
+        }
+        if( argc > 1 && !strcmp(argv[0],"-conf") ){
+            params->conf_file = argv[1];
             --argc;
             ++argv;
             continue;
         }
         if( argc > 1 && !strcmp(argv[0],"-in") ){
-            *inputfile = argv[1];
-            *mode = READ_FROM_FILE;
+            params->inputfile = argv[1];
+            params->mode = READ_FROM_FILE;
             inflag = 1;
             --argc;
             ++argv;
@@ -736,27 +1043,39 @@ int parseArgs(int argc, char **argv, int *subsetsize, int *mode, int *numit, cha
             continue;
         }
         if( !strcmp(argv[0],"-verbose") ){
-            *show_progress=1;
+            params->show_progress = 1;
+            continue;
+        }
+        if( !strcmp(argv[0],"-quick") ){
+            params->quick = 1;
             continue;
         }
         if( !strcmp(argv[0],"-branch") ){
-            *bench_type |= BENCH_BRANCH;
+            params->bench_type |= BENCH_BRANCH;
             continue;
         }
         if( !strcmp(argv[0],"-dcr") ){
-            *bench_type |= BENCH_DCACHE_READ;
+            params->bench_type |= BENCH_DCACHE_READ;
             continue;
         }
         if( !strcmp(argv[0],"-dcw") ){
-            *bench_type |= BENCH_DCACHE_WRITE;
+            params->bench_type |= BENCH_DCACHE_WRITE;
             continue;
         }
         if( !strcmp(argv[0],"-flops") ){
-            *bench_type |= BENCH_FLOPS;
+            params->bench_type |= BENCH_FLOPS;
             continue;
         }
         if( !strcmp(argv[0],"-ic") ){
-            *bench_type |= BENCH_ICACHE_READ;
+            params->bench_type |= BENCH_ICACHE_READ;
+            continue;
+        }
+        if( !strcmp(argv[0],"-vec") ){
+            params->bench_type |= BENCH_VEC;
+            continue;
+        }
+        if( !strcmp(argv[0],"-instr") ){
+            params->bench_type |= BENCH_INSTR;
             continue;
         }
 
@@ -765,12 +1084,12 @@ int parseArgs(int argc, char **argv, int *subsetsize, int *mode, int *numit, cha
     }
 
     // MODE INFO: mode 1 uses file; mode 2 uses all native events.
-    if(*mode == 1)
+    if(READ_FROM_FILE == params->mode)
     {
-        test = fopen(*inputfile, "r");
+        test = fopen(params->inputfile, "r");
         if(test == NULL)
         {
-            fprintf(stderr, "Could not open %s. Exiting...\n", *inputfile);
+            fprintf(stderr, "Could not open %s. Exiting...\n", params->inputfile);
             return -1;
         }
         fclose(test);
@@ -783,7 +1102,7 @@ int parseArgs(int argc, char **argv, int *subsetsize, int *mode, int *numit, cha
         return -1;
     }
 
-    // Make sure user specifies mode implicitly.
+    // Make sure user specifies mode explicitly.
     if(kflag == 0 && inflag == 0)
     {
         print_usage(name);
@@ -799,8 +1118,14 @@ int parseArgs(int argc, char **argv, int *subsetsize, int *mode, int *numit, cha
 
     // Write output files in the user-specified directory.
     dirlen = strlen(tmp);
-    *outputdir = (char*)malloc((2+dirlen)*sizeof(char));
-    len = snprintf( *outputdir, 2+dirlen, "%s/", tmp);
+    params->outputdir = (char*)malloc((2+dirlen)*sizeof(char));
+
+    if (NULL == params->outputdir) {
+        fprintf(stderr, "Failed to allocate memory.\n");
+        return -1;
+    }
+
+    len = snprintf( params->outputdir, 2+dirlen, "%s/", tmp);
     if( len < 1+dirlen )
     {
         fprintf(stderr, "Problem with output directory name.\n");
@@ -808,7 +1133,7 @@ int parseArgs(int argc, char **argv, int *subsetsize, int *mode, int *numit, cha
     }
 
     // Make sure files can be written to the provided path.
-    status = access(*outputdir, W_OK);
+    status = access(params->outputdir, W_OK);
     if(status != 0)
     {
         fprintf(stderr, "Permission to write files to \"%s\" denied. Make sure the path exists and is writable.\n", tmp);
@@ -830,13 +1155,17 @@ void print_usage(char* name)
     fprintf(stdout, "  Parameters \"-k\" and \"-in\" are mutually exclusive.\n");
     
     fprintf(stdout, "\nOptional:\n");
+    fprintf(stdout, "  -conf    <path>   Configuration file location.\n");
     fprintf(stdout, "  -verbose          Show benchmark progress in the standard output.\n");
+    fprintf(stdout, "  -quick            Skip latency tests.\n");
     fprintf(stdout, "  -n       <value>  Number of iterations for data cache kernels.\n");
     fprintf(stdout, "  -branch           Branch kernels.\n");
     fprintf(stdout, "  -dcr              Data cache reading kernels.\n");
     fprintf(stdout, "  -dcw              Data cache writing kernels.\n");
     fprintf(stdout, "  -flops            Floating point operations kernels.\n");
     fprintf(stdout, "  -ic               Instruction cache kernels.\n");
+    fprintf(stdout, "  -vec              Vector FLOPs kernels.\n");
+    fprintf(stdout, "  -instr            Instructions kernels.\n");
 
     fprintf(stdout, "\n");
     fprintf(stdout, "EXAMPLE: %s -in event_list.txt -out OUTPUT_DIRECTORY -branch -dcw\n", name);
