@@ -59,6 +59,9 @@ int _papi_hwi_num_errors = 0;
 hwi_presets_t user_defined_events[PAPI_MAX_USER_EVENTS];
 int user_defined_events_count = 0;
 
+THREAD_LOCAL_STORAGE_KEYWORD int _papi_rate_events_running = 0;
+THREAD_LOCAL_STORAGE_KEYWORD int _papi_hl_events_running = 0;
+
 /*****************************/
 /* Native Event Mapping Code */
 /*****************************/
@@ -111,31 +114,28 @@ _papi_hwi_free_papi_event_string() {
 	}
 	return;
 }
-// A place to keep the current papi event code so some component functions can fetch its value
-// The current event code can be stored here prior to component calls and cleared after the component returns
-static unsigned int papi_event_code = -1;
-static int papi_event_code_changed = -1;
+
 void
 _papi_hwi_set_papi_event_code (unsigned int event_code, int update_flag) {
-	INTDBG("new event_code: %#x, update_flag: %d, previous event_code: %#x\n", event_code, update_flag, papi_event_code);
+	INTDBG("new event_code: %#x, update_flag: %d, previous event_code: %#x\n", event_code, update_flag, _papi_hwi_my_thread->tls_papi_event_code);
 
 	// if call is just to reset and start over, set both flags to show nothing saved yet
 	if (update_flag < 0) {
-		papi_event_code_changed = -1;
-		papi_event_code = -1;
+		_papi_hwi_my_thread->tls_papi_event_code_changed = -1;
+		_papi_hwi_my_thread->tls_papi_event_code = -1;
 		return;
 	}
 
 	// if 0, it is being set prior to calling a component, if >0 it is being changed by the component
-	papi_event_code_changed = update_flag;
+	_papi_hwi_my_thread->tls_papi_event_code_changed = update_flag;
 	// save the event code passed in
-	papi_event_code = event_code;
+	_papi_hwi_my_thread->tls_papi_event_code = event_code;
 	return;
 }
 unsigned int
 _papi_hwi_get_papi_event_code () {
-	INTDBG("papi_event_code: %#x\n", papi_event_code);
-	return papi_event_code;
+	INTDBG("papi_event_code: %#x\n", _papi_hwi_my_thread->tls_papi_event_code);
+	return _papi_hwi_my_thread->tls_papi_event_code;
 }
 /* Get the index into the ESI->NativeInfoArray for the current PAPI event code */
 int
@@ -507,6 +507,8 @@ _papi_hwi_init_errors(void) {
 	/* 23 PAPI_ECOUNT */	_papi_hwi_add_error("Too many events or attributes");
 	/* 24 PAPI_ECOMBO */	_papi_hwi_add_error("Bad combination of features");
 	/* 25 PAPI_ECMP_DISABLED */_papi_hwi_add_error("Component containing event is disabled");
+    /* 26 PAPI_EDELAY_INIT */ _papi_hwi_add_error("Delayed initialization component");
+    /* 27 PAPI_EMULPASS */ _papi_hwi_add_error("Event exists, but cannot be counted due to multiple passes required by hardware");
 }
 
 int
@@ -560,7 +562,7 @@ _papi_hwi_native_to_eventcode(int cidx, int event_code, int ntv_idx, const char
 
   int result;
 
-  if (papi_event_code_changed > 0) {
+  if (_papi_hwi_my_thread->tls_papi_event_code_changed > 0) {
 	  result = _papi_hwi_get_papi_event_code();
 	  INTDBG("EXIT: papi_event_code: %#x set by the component\n", result);
 	  return result;
@@ -1344,7 +1346,8 @@ _papi_hwi_add_event( EventSetInfo_t * ESI, int EventCode )
 	if (cidx<0) {
 		return PAPI_ENOCMP;
 	}
-	if (_papi_hwd[cidx]->cmp_info.disabled) {
+	if (_papi_hwd[cidx]->cmp_info.disabled &&
+        _papi_hwd[cidx]->cmp_info.disabled != PAPI_EDELAY_INIT) {
 		return PAPI_ECMP_DISABLED;
 	}
 
@@ -1928,11 +1931,13 @@ int papi_num_components = ( sizeof ( _papi_hwd ) / sizeof ( *_papi_hwd ) ) - 1;
  * Routine that initializes all available components.
  * A component is available if a pointer to its info vector
  * appears in the NULL terminated_papi_hwd table.
+ * Modified to accept an arg: 0=do not init perf_event or 
+ * perf_event_uncore. 1=init ONLY perf_event or perf_event_uncore.
  */
 int
-_papi_hwi_init_global( void )
+_papi_hwi_init_global( int PE_OR_PEU )
 {
-        int retval, i = 0;
+        int retval, is_pe_peu, i = 0;
 
 	retval = _papi_hwi_innoculate_os_vector( &_papi_os_vector );
 	if ( retval != PAPI_OK ) {
@@ -1940,16 +1945,17 @@ _papi_hwi_init_global( void )
 	}
 
 	while ( _papi_hwd[i] ) {
-
+      is_pe_peu = 0;
+      if (strcmp(_papi_hwd[i]->cmp_info.name, "perf_event") == 0) is_pe_peu=1;
+      if (strcmp(_papi_hwd[i]->cmp_info.name, "perf_event_uncore") == 0) is_pe_peu=1;
 	   retval = _papi_hwi_innoculate_vector( _papi_hwd[i] );
 	   if ( retval != PAPI_OK ) {
 	      return retval;
 	   }
 
 	   /* We can be disabled by user before init */
-	   if (!_papi_hwd[i]->cmp_info.disabled) {
+	   if (!_papi_hwd[i]->cmp_info.disabled && (PE_OR_PEU == is_pe_peu)) {
 	      retval = _papi_hwd[i]->init_component( i );
-	      _papi_hwd[i]->cmp_info.disabled=retval;
 
 	      /* Do some sanity checking */
 	      if (retval==PAPI_OK) {
@@ -1992,7 +1998,6 @@ _papi_hwi_init_global_internal( void )
 
 	/* PAPI_hw_info_t struct */
 	memset(&(_papi_hwi_system_info.hw_info),0x0,sizeof(PAPI_hw_info_t));
-
 	return PAPI_OK;
 }
 
@@ -2312,7 +2317,7 @@ _papi_hwi_get_preset_event_info( int EventCode, PAPI_event_info_t * info )
 	   info->count = _papi_hwi_presets[i].count;
 
 	   _papi_hwi_derived_string( _papi_hwi_presets[i].derived_int,
-				     info->derived,  sizeof ( info->derived ) );
+				     info->derived,  sizeof ( info->derived )-1 );
 
 	   if ( _papi_hwi_presets[i].postfix != NULL )
 	      strncpy( info->postfix, _papi_hwi_presets[i].postfix,
@@ -2458,7 +2463,9 @@ _papi_hwi_native_name_to_code( const char *in, int *out )
 	// look in each component
 	for(cidx=0; cidx < papi_num_components; cidx++) {
 
-		if (_papi_hwd[cidx]->cmp_info.disabled) continue;
+		if (_papi_hwd[cidx]->cmp_info.disabled &&
+            _papi_hwd[cidx]->cmp_info.disabled != PAPI_EDELAY_INIT)
+            continue;
 
 		// if this component does not support the pmu
 		// which defines this event, no need to call it
@@ -2584,7 +2591,9 @@ _papi_hwi_get_native_event_info( unsigned int EventCode,
     cidx = _papi_hwi_component_index( EventCode );
     if (cidx<0) return PAPI_ENOCMP;
 
-    if (_papi_hwd[cidx]->cmp_info.disabled) return PAPI_ENOCMP;
+    if (_papi_hwd[cidx]->cmp_info.disabled &&
+        _papi_hwd[cidx]->cmp_info.disabled != PAPI_EDELAY_INIT)
+        return PAPI_ENOCMP;
 
     if ( EventCode & PAPI_NATIVE_MASK ) {
         // save event code so components can get it with call to: _papi_hwi_get_papi_event_code()
@@ -2730,3 +2739,59 @@ _papi_hwi_get_context( EventSetInfo_t * ESI, int *is_dirty )
 	}
 	return( ctx );
 }
+
+static int
+get_component_index(const char *name)
+{
+    int cidx;
+
+    for (cidx = 0; cidx < papi_num_components; ++cidx) {
+        if (strcmp(_papi_hwd[cidx]->cmp_info.name, name) == 0) {
+            break;
+        }
+    }
+
+    return cidx;
+}
+
+int
+_papi_hwi_enum_dev_type(int enum_modifier, void **handle)
+{
+    _papi_hwi_sysdetect_t args;
+    args.query_type = PAPI_SYSDETECT_QUERY__DEV_TYPE_ENUM;
+    args.query.enumerate.modifier = enum_modifier;
+
+    int cidx = get_component_index("sysdetect");
+    assert(cidx < papi_num_components);
+
+    return _papi_hwd[cidx]->user(0, &args, handle);
+}
+
+int
+_papi_hwi_get_dev_type_attr(void *handle, PAPI_dev_type_attr_e attr, void *value)
+{
+    _papi_hwi_sysdetect_t args;
+    args.query_type = PAPI_SYSDETECT_QUERY__DEV_TYPE_ATTR;
+    args.query.dev_type.handle = handle;
+    args.query.dev_type.attr = attr;
+
+    int cidx = get_component_index("sysdetect");
+    assert(cidx < papi_num_components);
+
+    return _papi_hwd[cidx]->user(0, &args, value);
+}
+
+int
+_papi_hwi_get_dev_attr(void *handle, int id, PAPI_dev_attr_e attr, void *value)
+{
+    _papi_hwi_sysdetect_t args;
+    args.query_type = PAPI_SYSDETECT_QUERY__DEV_ATTR;
+    args.query.dev.handle = handle;
+    args.query.dev.id = id;
+    args.query.dev.attr = attr;
+
+    int cidx = get_component_index("sysdetect");
+    assert(cidx < papi_num_components);
+
+    return _papi_hwd[cidx]->user(0, &args, value);
+}
