@@ -15,7 +15,7 @@
 
 
 #define NUM_INNER_LOCK  9
-#define PAPI_MAX_LOCK   (NUM_INNER_LOCK + PAPI_NUM_LOCK)
+#define PAPI_MAX_LOCK   (NUM_INNER_LOCK + PAPI_NUM_LOCK + PAPI_NUM_COMP)
 
 #include OSLOCK
 
